% This is a demo for the Adaptive Ranking Support Vector Machines (AdaRSVM) algorithm. 

% Please cite our paper, if you use this code for your project. Thanks.
% Andy J Ma, Pong C Yuen, Jiawei Li and Ping Li, 
%Cross-Domain Person Re-Identification Using Domain Adaptation Ranking SVMs, 
% IEEE Transactions on Image Processing (TIP), vol. 24, no. 5, pp. 1599-1613, 2015.

clear
clc

% set parameters
para.musqr = 10;
para.C = 1;
para.numNegPerImTar = 1;
para.numLabPerImSou = 1;

% parameters for viper dataset
para.Q = 300;    
num_im_train = 316;
num_im_test = 316;

% load souce domain model
load('source_prid.mat')

% load target domain data
load('target_viper.mat')
load('ind_split_rand_viper.mat')

for ind_run = 1:10       
    ind_test = ind_split_rand(ind_run,num_im_train+1:end);
    vecs_cama_test = vecs_cama(:,ind_test);
    vecs_camb_test = vecs_camb(:,ind_test);
    vecs_cama_train = vecs_cama(:, ind_split_rand(ind_run,1:num_im_train));
    vecs_camb_train = vecs_camb(:, ind_split_rand(ind_run,1:num_im_train));
    
    % gen negative samples
    data_tar_neg = zeros(size(vecs_cama_train,1), num_im_train*para.numNegPerImTar*2);
    for j = 1:num_im_train
        ind_neg = j+1:j+para.numNegPerImTar;
        ind_neg(ind_neg>num_im_train) = ind_neg(ind_neg>num_im_train) - num_im_train;
        data_tar_neg(:,(j-1)*para.numNegPerImTar*2+1:j*para.numNegPerImTar*2) = ...
            [abs(repmat(vecs_cama(:,j), [1 para.numNegPerImTar]) - vecs_camb(:,ind_neg)) ...
            abs(repmat(vecs_camb(:,j), [1 para.numNegPerImTar]) - vecs_cama(:,ind_neg))];
    end
    
    % train adaptive model
    [wt, miu_est] = AdaRSVM(ws, mean_sou_pos, mean_sou_neg, ...
        vecs_cama_train, vecs_camb_train, data_tar_neg, para);
        
    % testing
    
    num_im_cama_test = size(vecs_cama_test,2);
    num_im_camb_test = size(vecs_camb_test,2);
    
    mat_probs = zeros(num_im_cama_test,num_im_camb_test);
    mat_probt = zeros(num_im_cama_test,num_im_camb_test);
    
    for i = 1:num_im_test
        for j = 1:num_im_test
            mat_probs(i,j) = ws' * abs(vecs_cama_test(:,i) - vecs_camb_test(:,j));
            mat_probt(i,j) = wt' * abs(vecs_cama_test(:,i) - vecs_camb_test(:,j));
        end
    end   
       
    acc_cama = MyCMC(mat_probs(1:num_im_test,:)', 1:num_im_test);
    acc_camb = MyCMC(mat_probs(:,1:num_im_test), 1:num_im_test);    
    acc_camb(end+1:num_im_camb_test) = 1;    
    accs(ind_run,:) = (acc_cama + acc_camb) / 2;
    
    acc_cama = MyCMC(mat_probt(1:num_im_test,:)', 1:num_im_test);
    acc_camb = MyCMC(mat_probt(:,1:num_im_test), 1:num_im_test);    
    acc_camb(end+1:num_im_camb_test) = 1;    
    acct(ind_run,:) = (acc_cama + acc_camb) / 2;
    
    display(['run:' num2str(ind_run) ' , accs:' num2str(accs(ind_run,1))  ' , acct:' num2str(acct(ind_run,1))])
end
accmeans = mean(accs);
accmeant = mean(acct);

plot(accmeant, 'r')
hold on
plot(accmeans, '--', 'color', 'b')
legend('Ours', 'Source Domain RankSVM')